package jsystem;

import java.io.*;
import java.util.*;
import kjava.util.*;

import javax.microedition.rms.*;

// virtual file system based on RecordStores
// A file is a record with 
// HEADER bytes: Zero terminated filename
// + Databytes

public class Vfs{
    private static RecordStore files = null;
    private static Hashtable      fl = null;  // Directory list
    public  static final int HEADER  = 128;


    public Vfs(String name) throws Exception{
        files    = RecordStore.openRecordStore(name, true);
        fl       = readFileList();
    }

    // Return all data including header
    public static byte[] read(String fname) throws Exception{
         try{
            if(fl!=null && fl.containsKey(fname)){
                int id = ((Integer)fl.get(fname)).intValue();
                return files.getRecord(id);
            }
         }catch(Exception e){
         }
         throw new Exception("File " + fname + " could not be read.");
    }
	
	public static long length(String fname){
         try{
            if(fl!=null && fl.containsKey(fname)){
                int id = ((Integer)fl.get(fname)).intValue();
                return files.getRecord(id).length - HEADER;
			}else
				return 0;
		}catch(Exception e){
				return 0;
		}
	}
	
		

    private static boolean nameOK(String s){
         return s.length() > 0 && s.length() < HEADER;
    }


    public static void rename(String oldname, String newname) throws Exception{
         try{
            if(fl!=null && fl.containsKey(oldname) && nameOK(newname) ){
                // Update directory list
                Object obj = fl.get(oldname);
                fl.remove(oldname);
                fl.put(newname,obj);

                // Update Filesystem
                int id = ((Integer)obj).intValue();
                byte[] b = files.getRecord(id);

                for(int i=0; i<HEADER; i++)
                    b[i] = (byte)0;
                System.arraycopy(newname.getBytes(),0,b,0,newname.length());

                files.setRecord(id, b, 0, b.length);
            }
          }catch(Exception e){
         }
         throw new Exception("File " + oldname + " could not be renamed.");
    }
        

	public static synchronized void delete(String fname) throws IOException{
		// Check if file exists && remove it
		try{
			if(fl.containsKey(fname)){
 				int id = ((Integer)fl.get(fname)).intValue();
				files.deleteRecord(id);
				fl.remove(fname);
			}
		}catch(Exception e){
			throw new IOException("Could not delete "+fname);
		}
	}
		

    // Write/Overwrite record "fname"
    // if b == null, delete "fname"
    public static synchronized void write(byte[] b, int count, String fname) throws IOException{
        if( files == null )
             throw new IOException("VFS not initialized");
        if(!nameOK(fname))
              throw new IOException("Illegal Filename");
       
        try{
           // Write RecordStore, update directory
           if(fl==null){  // Create directory
               if( b== null)
                    return;
               fl = new Hashtable();
           }
           
           // Check if file exists && remove it
           if(fl.containsKey(fname)){
                int id = ((Integer)fl.get(fname)).intValue();
                files.deleteRecord(id);
                fl.remove(fname);
           }

           // Write new file
           if(b!=null){
                byte out[] = new byte[count+HEADER];
                for(int i=0; i<HEADER; i++)
                    out[i] = (byte)0;
                System.arraycopy(fname.getBytes(),0,out,0,fname.length());               
                System.arraycopy(b,0,out,HEADER,count); 
                int id = files.addRecord(out,0,out.length);
                fl.put(fname, new Integer(id));
           }
        }catch(Exception e){
           throw new IOException("Could not write "+ fname);
        }
     }


    public static Enumeration getFileList(){
        if(fl != null)
            return fl.keys();
        return null;
    }



    // Gets hashtable with filenames and recordIDs 
    private Hashtable readFileList(){
        if(files==null) return null;       
        try{
	    RecordEnumeration r = files.enumerateRecords(null, null, false);
            Hashtable h = new Hashtable();
            while(r.hasNextElement()) {
               int id   = r.nextRecordId();
               byte[] b = files.getRecord(id);
 
              // Read header              
               int i=0;
               while(b[i]!=(byte)0) i++;
               if(i>0 && i<HEADER){
                   h.put(new String(b,0,i), new Integer(id));
               }
            }
            r.destroy();
            return h;
        }catch (Exception e) {
	}
        return null;
    }


    public static long lastModified(){
      if(files!=null){
         try{
             return files.getLastModified();
         }catch(Exception e){  
         } 
      }                  
      return 0;
    }

        
}

